<?php

class glErrorHandler {

protected $current_error     = 0;
protected $current_parameter = 1;
protected $error_types       = null;
static    $instance          = null;

public function getInstance()
{
    if (self::$instance === null) { self::$instance = new glErrorHandler(); }
    return self::$instance;
}

public function handleError($error_type, $error_message, $file, $line, & $context)
{
    if (!($error_type & error_reporting())) { return; }
    // TODO: nesting ob_levels is pretty faulty
    $buffers = array ();
    while (ob_get_level()) {
        $buffer = ob_get_contents();
        array_unshift($buffers, $buffer);
        ob_end_clean();
    }
    $this->printCSS();
    $this->printJs();
    $this->error_types = array (  1 => "Php Error",        2 => "Php Warning",         4 => "Parsing Error",   8 => "Php Notice",
                                 16 => "Core Error",      32 => "Core Warning",       64 => "Compile Error", 128 => "Compile Warning",
                                256 => "Php User Error", 512 => "Php User Warning", 1024 => "Php User Notice");
    $error = array ('type' => $error_type, 'message' => $error_message, 'file' => $file, 'line' => $line, 'context' => $context,);
    $error['backtrace'] = debug_backtrace();
//    array_shift($error['backtrace']);
    $this->printError($error);
    foreach ($buffers as $buffer) {
        ob_start();
        print $buffer;
    }
}

private function printError($error)
{
                printf('<ul id="gl.error_%d" class="gl-error">', $this->current_error);
    echo "\n";  printf('<li><strong>Error Type:</strong> %s</li>', $this->error_types[$error['type']]);
    echo "\n";  printf('<li><strong>Message:</strong> %s</li>', $error['message']);
    echo "\n";

    if (isset ($error['file']) and isset ($error['line'])) {
                printf('<li><strong>File/Line:</strong> %s/%d</li>', hidePath($error['file']), $error['line']);
        echo "\n";
    }
                printf('<li><strong>Context (%d):</strong> ', count($error['context']));
    echo "\n";  printf('<a href="javascript:toggleErrorVisibility(\'gl.error_context_%d\');">[toggle]</a>', $this->current_error);
    echo "\n";  printf('<pre id="gl.error_context_%d" class="Parameter" style="display:none;">', $this->current_error);
                print htmlentities(value_dump($error['context'], 10));
                print '</pre>';
                print '</li>';
                printf('<li><strong>Backtrace (%d):</strong> ', count($error['backtrace']));
    echo "\n";  printf('<a href="javascript:toggleErrorVisibility(\'gl.error_backtrace_%d\');">[toggle]</a>', $this->current_error);
    echo "\n";  printf('<div id="gl.error_backtrace_%d" class="Backtrace" style="display:none;">', $this->current_error);
    echo "\n";  print '<ul>';
    echo "\n";

    foreach ($error['backtrace'] as $s => $step) {
                print '<li><span class="Symbol">';
        if (isset ($step['class'])) { printf('%s%s', $step['class'], $step['type']); }
                printf('%s', $step['function']);
                print '</span>';
                print '<span class="Parameters">(';
        for ($i = 0; $i < @ count(@ $step['args']); $i ++) {
            printf('<a href="javascript:toggleErrorVisibility(\'gl.error_backtrace_%d_step_%d_parameter_%d\');">%s</a>', $this->current_error, $s, $i, gettype($step['args'][$i]));
            if ($i < count($step['args']) - 1) { echo ', '; }
        }
                    print ')</span>';
        echo "\n";  printf('<a href="javascript:toggleErrorVisibility(\'gl.error_backtrace_%d_step_%d_source\');">[toggle source]</a>', $this->current_error, $s);
                    print '<ul class="Location">';
        echo "\n";

        if (isset ($step['file'])) {
            printf('<li><strong>file:</strong> %s</li>', hidePath($step['file']));
            echo "\n";
        }
        if (isset ($step['line'])) {
            printf('<li><strong>line:</strong> %d</li>', $step['line']);
            echo "\n";
        }
        print '</ul>';
        echo "\n";
        for ($i = 0; $i < @ count(@ $step['args']); $i ++) {
            printf('<pre class="Parameter" id="gl.error_backtrace_%d_step_%d_parameter_%d" style="display:none;">[param %d] %s</pre>', $this->current_error, $s, $i, $i, htmlentities(value_dump($step['args'][$i], 4)));
        }
        printf('<pre id="gl.error_backtrace_%d_step_%d_source"  class="SourceContext" style="display:none;">', $this->current_error, $s);
        echo "\n";

        if (isset ($step['file']) and isset ($step['line'])) {
            print $this->getSourceContext($step['file'], $step['line']);
        } else {
            print "Sourcecontext cannot be shown - ";
        }
        print '</pre>';
        print '</li>';
        echo "\n";
    }
                print '</ul>';
    echo "\n";  print '</div>';
    echo "\n";  print '</li>';
    echo "\n";  print '</ul>';
    echo "\n";
    $this->current_error++;
}

private function getSourceContext($file, $line, $before = 10, $after = 10)
{
    $data  = file($file);
    $count = count($data) - 1;
    //count which lines to display
    $start = $line - $before;
    if ($start < 1) { $start = 1; }
    $end   = $line + $after;
    if ($end > $count) { $end = $count +1; }
    $result = '';
    for ($i = $start; $i <= $end; $i ++) {
        if ($i == $line) { $result .= '<span class="ErrorLineNumber">';
        } else           { $result .= '<span class="LineNumber">'; }
        $result .= str_repeat('0', strlen($end) - strlen($i) + 1).$i;
        $result .= '</span> ';
        $result .= htmlentities($data[$i -1]);
    }
    return $result;
}

private function printJs()
{
    print <<<'JS'
    <script type="text/javascript">
    function toggleErrorVisibility(object) {
      styleObj = document.getElementById(object).style;
      if (styleObj.display=='none') {
        styleObj.display = '';
      } else {
        styleObj.display = 'none';
      }
    }
    </script>
JS;
}

private function printCSS()
{
    print <<<'CSS'
    <style type="text/css" media="screen">
    .gl.error { text-align: left }
    .ErrorLineNumber { background-color: red; color: white; }
    .LineNumber { color: orange; }
    </style>
CSS;
}

}  // end of glErrorHandler
?>