<?php
/* ---------------------- ---------------------- ---------------------- node ----------------------
   glPage class contains utilities to render data
 */
class glPage {
/* this class renders data acquired by vertices controlled by the canvas
 */
public           $name       = '';                          // name of the node in lowercase
public           $localNSI   = '';                          // local style NameSpace identifier
public           $layout     = array();                     // a list of consecutive containers; used for sequencing them in the final node
public           $skins      = array();                     // registered skins as scoped styles
public           $resources  = array();                     // registered resources, will be injected in head or foot 
public           $templates  = array();                     // all instantiated templates, will be finished on render
public           $snippets   = array();                     // collection of all content snippets, ready to go into a Template
public           $messages   = array();                     // all caught messages to display

public    static $pageLayout = '';                          // string containing outline (layout of root page)
public    static $pageTitle  = '';
public    static $cssSelectorNSI = 'gl';                    // global style NameSpace Identifier
protected static $nodes      = array();                     // all instantiated node objects, indexed on name

public  function setOutline($layout)         { self::$pageLayout = $layout; }
public  function setCssSelectorNS($prefix)   { self::$cssSelectorNSI  = $prefix; }
public  function setStyleNS($prefix)         { $this->localNSI = $prefix; }
public  function setPageTitle($title)        { self::$pageTitle  = $title; }
public  function addLayout($string)          { $this->layout = $this->buildLayout($string, $this->localNSI); }
public  function addSnippet($name, $content) { $this->snippets[$name] = $content; }
public  function getSnippet($name)           { return $this->snippets[$name]; }
public  function addResource($id, $what)     { $GLOBALS['_SCRIPTS']->setJavaScriptFile($id, $what, false, 100); } // js resource in head seq 100
public  function addLibrary($which)          { $GLOBALS['_SCRIPTS']->setJavaScriptLibrary($which, false); }       // in head
public  function addScript($where, $content) { $GLOBALS['_SCRIPTS']->setJavaScript($content, true, false); }      // wrap a tag, in head
public  function addStyle($where, $content)  { $GLOBALS['_SCRIPTS']->setCSS($content); }
public  function addSkin($name, $class)      { $this->skins[$name][] = $class; }
private function __construct($name)          { $this->name  = strtolower($name); $this->localNSI = self::$cssSelectorNSI; }

private function buildLayout($layout, $myNSI) 
{
    $proto = array();
    foreach (split(',', $layout) as $string)
        {   list($name, $id, $class) = split(':', $string);
            $id      = (empty($id))    ? $myNSI.'.'.$name: $id;
            $class   = (empty($class)) ? $myNSI.'-'.$name: $class;
            $proto[] = array('name' => $name, 'id' => $id, 'class' => $class, 'namespace' => $myNSI);
        }
    return $proto;
}
public function getPage($parentName)
{
    return self::getNode($parentName);
}

public function getNode($parentName) 
{
    $name  = strtolower($parentName);
    if (! isset(self::$nodes[$name])) {
        $node = new glPage($name);
        self::addNode($node);
    }
    if (! self::$nodes[$name] instanceOf glPage) {
        throw(new Exception('#glPage.Failure#' . "Invalid node on [{$parentName}]"));
    }
    return self::$nodes[$name];
}

protected function addNode(glPage $node) 
{
    $name = $node->name;
    if (isset(self::$nodes[$name])) {
        $names = implode(',', array_keys(self::$nodes));
        throw(new Exception('#glPage.Failure#' . "Constructed node on [{$name}] is duplicated in [{$names}]"));
    }
    self::$nodes[$name] = $node;
}

public function addTemplate($root, $context)
{
    $template = new Template($root);
    $this->templates[$context][] = $template;
    return $template;
}

public function addMessage($msg, $context)
{
    $this->messages[$context][] = COM_showMessageText($msg, 'Message from '.$this->name);
//  return COM_showMessage($msg, $context);
}

public function getMessages($context = '')
{
    if (empty($context)) { return $this->messages; }
    else                 { return $this->messages[$context]; }
}

public function getMessagesContent($context = '')
{
    $messages = $this->getMessages($page['name']);
    $content  = '';
    if (!empty($messages)) {
        foreach ($messages as $seq => $message) {
            $content .= $message;
        }
    }
    return $content;
}

public function setContent($what, $snippet, $append = false)
{
    if ($append) { $this->addContent($what, $snippet); }
    else         { $this->addSnippet($what, $snippet); }
}

public function addContent($what, $snippet)
{
    $this->addSnippet($what, $this->getSnippet($what) . $snippet);
}

public function getContent($what = '')
{
    $content = '';
    foreach ($this->layout as $id => $page) {
        $that     = $page['name'];
        $content .= "\n<!-- [{$what}] [{$that}] [ns:{$page['namespace']}]-->\n";
        if (empty($what) or $what == $that or $what == $this->name) {
            $content   .= $this->startBlock($what, $page);
            $content   .= $this->getMessagesContent($that);
            $myContent  = $this->getSnippet($that);
            if (isset(self::$nodes[$that])) { 
//              $myContent .= $this->getPageContent($that); 
                $myContent .= self::$nodes[$that]->getContent($that); 
            }
            if (count($this->templates[$that])) {
                foreach ($this->templates[$that] as $seq => $template) {
                    $template->set_var("gl.$that", $myContent);
                    $content .= $template->parse('glPage', "gl.$that");
                }
            } else {
                $content .= $myContent;
            }
            $content .= $this->endBlock($what, $page);
        }
    }
    return $content;
}

private function startBlock($what, $page)
{
    if (!empty($page['id'])) {
        if (isset($this->skins[$what])) { $class = ' ' . implode(' ', $this->skins[$what]); }
        return "<div id='{$page['id']}' class='{$page['class']}{$class}' title='{$page['name']} of {$what}' >";
    }
    return '';
}

private function endBlock($what, $page)
{
    if (!empty($page['id'])) {
        return "</div>";
    }
    return '';
}

public function hookTemplate($purpose='', $context='')
{
    $this->assignTemplate($purpose, $context);
}

protected function assignTemplate($purpose = 'default', $context ='')
{
    if (isset($this->template)) { return; }

    if (empty($purpose))        { $purpose = 'default'; }
    if (empty($context))        { $context = self::$compliance; }

    if (empty($this->tRoot))    { $tPath  = $this->getTemplateRoot($context, $purpose); }
    else                        { $tPath  = self::$layout . $this->tRoot; }

    if (!isset($this->template)) { $this->template = new Template($tPath); }
    $this->template->set_file($this->tFiles);
}

/*
 *  acts as facade to proxy @todo 
 */
protected function getTemplateRoot($context = 'public', $purpose ='default')
{
    global $_CONF;
    $plugin     = $this->name;
    $glPageConf = '_' .strtoupper($plugin). '_CONF'; // @todo find the plugin Name or call canvas
    global $$glPageConf;
    $glPageConf = $$glPageConf;

    if ($context === 'admin') {
        $configured_tDir = $glPageConf['templates_admin'];
        $fallback = 'admin';
    } else {
        $configured_tDir = $glPageConf['templates_user'];
        $fallback = 'public';
    }

    switch ($configured_tDir) {
        case 'theme' :
            $tPath = $_CONF['path_layout'] . $glPageConf['themespath'] .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
            break;
        case 'custom' :
            $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/custom/templates/' .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
            break;
        case 'standard':
        default        :
            $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$context;
            if ($context <> 'admin')     { $tPath .= '/' . $purpose; }
            if (!is_dir($tPath)) {
                $tPath = $_CONF['path'] . 'plugins/' .$plugin. '/templates/' .$fallback;
                if ($context <> 'admin') { $tPath .= '/default'; }
            }
    }
    return $tPath;
}

/* @TODO check COM_ and names ß*/
public function renderPage($preference = '' , $display ='', $custom = '' )
{
    global $_CONF;

//    $glPageConf = '_'.strtoupper($this->pi_name)."_CONF";
//    global $$glPageConf;
//    $glPageConf = $$glPageConf;

	if  ($preference=="_admin" OR $preference==""){
		$layout=$glPageConf['layout'.$preference];
    } else {
    	$layout=$preference;
    }
    $information = array();
    if (!empty($custom)) {
        $information['custom'] = $custom;
    }
    switch ($layout) {
        case 'standard':
            $information['what']            = 'menu';
            $information['rightblock']      = false;
            break;
        case 'leftrightblocks':
            $information['what']            = 'menu';
            $information['rightblock']      = true;
            break;
        case 'rightblocks':
            $display .= COM_siteFooter(true, $custom);
            break;
        case 'leftblocks':
            $information['what']            = 'menu';
            $_CONF['show_right_blocks']     = 0;
            $_CONF['left_blocks_in_footer'] = 0;
            $information['rightblock']      = false;
            break;
        case 'noblocks':
            $information['what']            = 'none';
            $information['rightblock']      = false;
            $_CONF['show_right_blocks']     = 0;
            $_CONF['left_blocks_in_footer'] = 0;
            break;
        case 'blankpage':
            break;
        default:
    }
        $information['pagetitle'] = self::$pageTitle;

        if (empty($display)) { $display = self::getPageContent(); }

        if (COM_versionCompare(VERSION, "2.0.0",  '>=')) {
            $display  = COM_createHTMLDocument($display, $information);
        }else{
            $display  = COM_siteHeader($information['what'], $information['pagetitle']) . $display;
            $display .= COM_siteFooter($information['rightblock']);
        }
    return $display;
    }

public function getOutline($what = '')
{
    if (empty($what))   { $proto = self::buildLayout(self::$pageLayout, self::$cssSelectorNSI); }
    else                { $proto = $this->layout; }
    if (!count($proto) ) { $keys = '_'; }
    foreach ($proto as $seq => $page) {
        $that = $page['name'];
        if (empty($what) or $what == $that or $what == $this->name) {
            $keys .= $comma . $that . ':';
            if (isset(self::$nodes[$that])) { 
                $keys .= self::$nodes[$that]->getOutline($that); 
            } else {
                $keys .= '?';
            }
            $comma = ',';
        }
    }
    return $keys;
}

public function getPageContent($what = '')
{
    if (empty($what))   { $proto = self::buildLayout(self::$pageLayout, self::$cssSelectorNSI); }
    else                { $proto = $this->layout; }
    $content   = '';
    foreach ($proto as $seq => $page) {
        $that = $page['name'];
        if (empty($what) or $what == $that or $what == $this->name) {
            $content .= "\n<!-- node:".$that." ns:".$page['namespace']." -->\n" . self::startBlock($that, $page);
            if (isset(self::$nodes[$that])) { 
                $content .= self::$nodes[$that]->getContent($that); 
            }
            $content .= self::endBlock($that, $page) . "\n<!-- end:" .$that. " -->\n";
        }
    }
    return $content;
}

/*
 * fix new lines in raw text
 */
public function fixNewLines($text) 
{
    return str_replace(array("\r\n","\n\r","\r","\n"), '<br'.XHTML.'>', $text);
}

/*
 * scan for autolinks and fix them in raw text
 */
public function fixAutoLink($text) 
{
    $patterns     = array("/(https?|ftp)(:\/\/[[:alnum:]\+\$\;\?\.%,!#~*\/:@&=_-]+)/i");
    $replacements = array("<a href=\"\\1\\2\" target=\"_blank\">\\1\\2</a>");
    return preg_replace($patterns, $replacements, $text);
}

} // end of glPage
 ?>